package com.muyu.eventdriven.server.impl;

import com.alibaba.fastjson2.JSON;
import com.muyu.eventdriven.consumer.KafkaConsumers;
import com.muyu.eventdriven.domain.EventTacticsManage;
import com.muyu.eventdriven.domain.VehicleData;
import com.muyu.eventdriven.domain.VehicleKafka;
import com.muyu.eventdriven.domain.rest.Result;
import com.muyu.eventdriven.server.EventInfoService;
import com.muyu.eventdriven.tactics.EventTactics;
import lombok.extern.log4j.Log4j2;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.clients.consumer.ConsumerRecords;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.time.Duration;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;

/**
 * @ClassName EventInfoServiceImpl
 * @Description 事件实现层
 * @Author Yunfei.Du
 * @Date 2024/6/18 上午9:35
 */
@Service
@Log4j2
public class EventInfoServiceImpl implements EventInfoService {
    private static Map<String,KafkaConsumer> kafkaConsumerMap = new HashMap<>();

    private static List<String> classNameList = new ArrayList<>((Arrays.asList(
            "StorageEvent",
            "ElectronicFenceEvent",
            "FaultAlarmEvent",
            "RealTimeDataEvent",
            "IndexWarningEvent")));

    @Autowired
    private KafkaConsumers kafkaConsumers;
    @Autowired
    private RedisTemplate<String,String> redisTemplate;
    @Autowired
    private EventTacticsManage eventTacticsManage;
    @Autowired
    private ApplicationContext applicationContext;

    /**
     * 创建 Kafka 消费者实例，用于消费与特定车辆 VIN 相关联的 Kafka 主题。
     *
     * @param vehicleVin 车辆的唯一识别码（VIN），用于确定要消费的主题。
     */
    @Override
    public void creatKafkaConsumer(String vehicleVin) {
        // 从 Redis 中获取车辆 Kafka 配置信息
        Object o = redisTemplate.opsForHash().get("vehicleKafka", vehicleVin);
        VehicleKafka vehicleKafka = JSON.parseObject ( o.toString (), VehicleKafka.class );

        // 将车辆 VIN 添加到 Redis 列表中，标记该 VIN 有活跃的消费者
        redisTemplate.opsForList().rightPush(vehicleVin,"");
        // 设置 VIN 在 Redis 中的过期时间，避免消费者长时间不活跃但仍被保留
        redisTemplate.expire(vehicleVin, 10, TimeUnit.HOURS);
        // 检查是否存在已为当前车辆配置的 Kafka 消费者
        if (!kafkaConsumerMap.containsKey(vehicleKafka.getConsumerName() + "-" + vehicleKafka.getPartitions())) {
            // 根据车辆 Kafka 配置创建一个新的 Kafka 分区消费者
            KafkaConsumer kafkaConsumer = kafkaConsumers.kafkaConsumer(vehicleKafka);
            // 将新创建的消费者添加到映射中，键为消费者名称和分区组合
            new Thread ( () -> {
                kafkaConsumerMap.put(vehicleKafka.getConsumerName()+"-"+vehicleKafka.getPartitions(),kafkaConsumer);
                // 持续检查消费者映射中是否存在当前车辆的消费者，存在则处理消息
                while (true){
                    // 使用 ConcurrentHashMap 来保证线程安全
                    ConcurrentHashMap<String, ArrayList<VehicleData>> stringListHashMap = new ConcurrentHashMap<>();
                    // 从 Kafka 拉取消息
                    // 拉取消息
                    ConsumerRecords<String, String> msg = kafkaConsumer.poll(Duration.ofSeconds(100));
                    // 处理拉取到的消息，将消息按车辆事件类型分类
                    stringListHashMap = getVehicleData(msg, stringListHashMap);
                    // 对于每种车辆事件类型，异步处理相关消息
                    stringListHashMap.forEach((key,value) -> {
                        // 从 Redis 中获取车辆事件处理类的列表
//                    String vehicleEventString = redisTemplate.opsForHash().get(RedisConstants.VEHICLE_EVENT, key).toString();
                        String vehicleEventString = "1,2,3,4,5";
                        for (String str : vehicleEventString.split(",")) {
                            CompletableFuture.runAsync(() -> {
                                if (vehicleEventString.contains ( str )){
                                    applicationContext.getBean(classNameList.get(Integer.parseInt(str)), EventTactics.class).eventManage(key,value);
                                }

                            });
                        }
                    });
                }
            }).start ();
        }
    }

    public ConcurrentHashMap<String, ArrayList<VehicleData>> getVehicleData(ConsumerRecords<String, String> msg,ConcurrentHashMap<String, ArrayList<VehicleData>> stringListHashMap) {
        for (ConsumerRecord<String, String> consumerRecord : msg) {
            try {
                VehicleData vehicleData = JSON.parseObject(consumerRecord.value(), VehicleData.class);
                // 使用 compute 方法简化数据添加逻辑
                stringListHashMap.compute(vehicleData.getVin(), (vin, list) -> {
                    if (list == null) {
                        list = new ArrayList<>();
                    }
                    list.add(vehicleData);
                    return list;
                });
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return stringListHashMap;
    }

    @Override
    public Result closeKafkaConsumer(String vin) {
        redisTemplate.delete(vin);
        return Result.success("释放消费者");
    }
}
