package com.muyu.eventdriven.tactics.system;

import com.alibaba.fastjson.JSON;
import com.github.benmanes.caffeine.cache.Cache;
import com.muyu.eventdriven.constants.FaultCodeConstants;
import com.muyu.eventdriven.constants.RabbitConstants;
import com.muyu.eventdriven.constants.RedisConstants;
import com.muyu.eventdriven.domain.RabbitFalut;
import com.muyu.eventdriven.domain.VehicleData;
import com.muyu.eventdriven.domain.VehicleFaultStatus;
import com.muyu.eventdriven.tactics.EventTactics;
import lombok.extern.log4j.Log4j2;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

/**
 * @ClassName FaultAlarmEvent
 * @Description 故障报警
 * @Author Yunfei.Du
 * @Date 2024/6/20 上午10:58
 */
@Service("FaultAlarmEvent")
@Log4j2
//@RequiredArgsConstructor
public class FaultAlarmEvent implements EventTactics {
    @Autowired
    private Cache<String, Object> caffeineCache;

    @Autowired
    private RedisTemplate<String,String> redisTemplate;

    @Autowired
    private RabbitTemplate rabbitTemplate;


    @Override
    public void eventManage(VehicleData vehicleData) {
        log.info("车辆{}执行故障报警事件",vehicleData.getVin());
    }

    @Override
    public void eventManage(String vin, List<VehicleData> vehicleDataList) {
        vehicleDataList.stream().forEach(vehicleData -> {
            //车辆状态
            if (vehicleData.getVehicleStatus() == 0){
                hasLocalCache(vehicleData, FaultCodeConstants.VEHICLESTATUS);
            }
            //充电状态
            if (vehicleData.getChargingStatus() == 0){
                hasLocalCache(vehicleData, FaultCodeConstants.CHARGINGSTATUS);
            }
            //运行状态
            if (vehicleData.getOperatingStatus() == 0){
                hasLocalCache(vehicleData, FaultCodeConstants.OPERATINGSTATUS);
            }
            //SOC
            if (vehicleData.getSocStatus() == 0){
                hasLocalCache(vehicleData, FaultCodeConstants.SOCSTATUS);
            }
            //可充电储能装置工作状态
            if (vehicleData.getChargingEnergyStorageStatus() == 0){
                hasLocalCache(vehicleData, FaultCodeConstants.CHARGINGENERGYSTORAGESTATUS);
            }
            //驱动电机状态
            if (vehicleData.getDriveMotorStatus() == 0){
                hasLocalCache(vehicleData, FaultCodeConstants.DRIVEMOTORSTATUS);
            }
            //定位是否有效
            if (vehicleData.getPositionStatus() == 0){
                hasLocalCache(vehicleData, FaultCodeConstants.POSITIONSTATUS);
            }
            //EAS(汽车防盗系统)状态
            if (vehicleData.getEasStatus() == 0){
                hasLocalCache(vehicleData, FaultCodeConstants.EASSTATUS);
            }
            //PTC(电动加热器)状态
            if (vehicleData.getPtcStatus() == 0){
                hasLocalCache(vehicleData, FaultCodeConstants.PTCSTATUS);
            }
            //EPS(电动助力系统)状态
            if (vehicleData.getEpsStatus() == 0){
                hasLocalCache(vehicleData, FaultCodeConstants.EPSSTATUS);
            }
            //ABS(防抱死)状态
            if (vehicleData.getAbsStatus() == 0){
                hasLocalCache(vehicleData, FaultCodeConstants.ABSSTATUS);
            }
            //MCU(电机/逆变器)状态
            if (vehicleData.getMcuStatus() == 0){
                hasLocalCache(vehicleData, FaultCodeConstants.MCUSTATUS);
            }
            //动力电池加热状态
            if (vehicleData.getHeatingStatus() == 0){
                hasLocalCache(vehicleData, FaultCodeConstants.HEATINGSTATUS);
            }
            //动力电池当前状态
            if (vehicleData.getBatteryStatus() == 0){
                hasLocalCache(vehicleData, FaultCodeConstants.BATTERYSTATUS);
            }
            //动力电池保温状态
            if (vehicleData.getBatteryInsulationStatus() == 0){
                hasLocalCache(vehicleData, FaultCodeConstants.BATTERYINSULATIONSTATUS);
            }
            //DCDC(电力交换系统)状态
            if (vehicleData.getDcdcStatus() == 0){
                hasLocalCache(vehicleData, FaultCodeConstants.DCDCSTATUS);
            }
            //CHG(充电机)状态
            if (vehicleData.getChgStatus() == 0){
                hasLocalCache(vehicleData, FaultCodeConstants.CHGSTATUS);
            }

        });
        log.info("车辆{}执行故障报警事件",vin);
    }

    /**
     * 检查车辆故障数据是否已在本地缓存中，并根据情况更新缓存和发送故障状态消息。
     * 如果缓存中已存在车辆的故障数据，则不进行任何操作。
     * 如果缓存中不存在车辆的故障数据，则将数据存入缓存，并发送车辆故障状态消息。
     *
     * @param vehicleData 车辆数据，包含车辆vin和行驶路线等信息。
     * @param faultCode 故障代码，用于标识具体的故障类型。
     */
    public void hasLocalCache(VehicleData vehicleData,String faultCode){
        Object o = caffeineCache.get(RedisConstants.VEHICLE_FAULT_KEY+vehicleData.getVin()+":" + faultCode, key -> vehicleData.getDrivingRoute());
        if (o.toString().equals(vehicleData.getDrivingRoute())){
            RabbitFalut rabbitFalut = new RabbitFalut();
            rabbitFalut.setStartTime(new Date());
            rabbitFalut.setVin(vehicleData.getVin());
            rabbitFalut.setFaultCode(faultCode);
            rabbitTemplate.convertAndSend(RabbitConstants.EXCHANGE_STATUS,RabbitConstants.QUEUE_STATUS_NORMAL, JSON.toJSONString(rabbitFalut),message ->{
                message.getMessageProperties().setMessageId( UUID.randomUUID().toString());
                //设置消息延迟时间为5秒
                message.getMessageProperties().setDelay(5000);
                log.info ( "消息发送成功" );
                return message;
            });
        }
        redisTemplate.opsForValue().set(RedisConstants.VEHICLE_FAULT_KEY+vehicleData.getVin()+":" + faultCode,vehicleData.getDrivingRoute(),10, TimeUnit.SECONDS);
    }
}
